from contextvars import ContextVar

from starlette.datastructures import Headers
from starlette.types import ASGIApp, Receive, Scope, Send

from web.exceptions import ProcessingException

USER_CONTEXT_KEY = "user_context"


class UserContext:
    id: str
    username: str
    session_id: str
    roles: list[str]


user_context_ctx_var: ContextVar[UserContext] = ContextVar(USER_CONTEXT_KEY, default=UserContext())


def get_user_context() -> UserContext:
    return user_context_ctx_var.get()


def parse_headers(headers: Headers):
    id = headers.get("X-MACUKA-ID")
    username = headers.get("X-MACUKA-USERNAME")
    roles = headers.get("X-MACUKA-ROLES")
    cookies = headers.get('cookie').split(';')

    if roles is None or username is None or roles is None:
        return None

    session_id = None
    for cookie in cookies:
        typed_cookie: str = cookie
        if typed_cookie.upper().count("X_MACUKA_SESSION") == 1:
            session_id = typed_cookie.split('=')[1]
            break
    if session_id is None:
        raise ProcessingException('session it not found')

    context = UserContext()
    context.id = id.split(":")[0]
    context.username = username.split(":")[0]
    context.roles = roles.split(":")[0].split(",")
    context.session_id = session_id
    return context


class UsernameMiddleware:
    def __init__(
            self,
            app: ASGIApp,
    ) -> None:
        self.app = app

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        if scope["type"] not in ["http", "websocket"]:
            await self.app(scope, receive, send)
            return

        headers = Headers(scope=scope)

        context = parse_headers(headers)

        if context is None:
            await self.app(scope, receive, send)
            return

        user_context = user_context_ctx_var.set(context)

        await self.app(scope, receive, send)

        user_context_ctx_var.reset(user_context)



import os
from abc import ABC, abstractmethod


class Database(ABC):
    """
    Database context manager
    """

    def __init__(self, driver) -> None:
        self.driver = driver

    @abstractmethod
    def connect_to_database(self):
        raise NotImplementedError()

    def __enter__(self):
        self.connection = self.connect_to_database()
        self.cursor = self.connection.cursor()
        return self

    def __exit__(self, exception_type, exc_val, traceback):
        self.cursor.close()
        self.connection.close()

    def get_connection_params(self):
        username = os.getenv("DB_USERNAME", "backend")
        password = os.getenv("DB_PASSWORD", "backend")
        db_url: str = os.getenv("DB_URL", "jdbc:postgresql://localhost:5432/backend")
        db_name = db_url.split("/")[-1]
        db_host_port = db_url.split("//")[1].split("/")[0]
        db_host, db_port = db_host_port.split(":")

        return username, password, db_name, db_host, db_port



from web.dto import CommonDto
from pydantic import Field


class ProcessingException(Exception):
    def __init__(self, message):
        self.message = message


class RestException(CommonDto):
    type: str
    message: str = Field(None)
    details: list[dict] = Field(None)



from web.database import Database
import os
from pymongo import MongoClient

class MongoDatabase(Database):
    def __init__(self):
        super().__init__(None)
    def connect_to_database(self):
        db_uri, db_name = self.get_connection_params()
        return MongoClient(db_uri)

    def get_connection_params(self):
        db_uri = os.getenv('MONGO_URI')
        db_name = os.getenv('MONGO_DB')
        return db_uri, db_name

    def __enter__(self):
        db_uri, db_name = self.get_connection_params()
        self.client = MongoClient(db_uri)
        self.db = self.client[db_name]
        return self.db

    def __exit__(self, exception_type, exc_val, traceback):
        self.client.close()
